@@ -3,13 +3,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <limits.h>
 #include <assert.h>
 #include <locale.h>
 #include <langinfo.h>
 #include <nl_types.h>
 #include <stdint.h>
 
-#define __CTYPE_HAS_8_BIT_LOCALES
 #include "c8tables.h"
 
 
@@ -47,6 +47,7 @@ typedef struct {
 	unsigned char lc_numeric_row;
 	unsigned char lc_monetary_row;
 	unsigned char lc_messages_row;
+	unsigned char lc_ctype_row;
 #if CATEGORIES != 6
 #error unsupported CATEGORIES value
 #endif
@@ -68,6 +69,7 @@ static void do_lc_numeric(void);
 static void do_lc_monetary(void);
 
 static void do_lc_messages(void);
+static void do_lc_ctype(void);
 
 
 static FILE *fp;
@@ -100,6 +102,9 @@ static void do_locale_names(void)
 		fprintf(ofp, "static const unsigned char __locales[%d];\n", (3 + CATEGORIES));
 		fprintf(ofp, "static const unsigned char __locale_names5[5];\n");
 	} else {
+		if (default_utf8) {
+			fprintf(ofp, "#define __CTYPE_HAS_UTF_8_LOCALES\t\t\t1\n");
+		}
 		fprintf(ofp, "#define CATEGORIES\t\t\t%d\n", CATEGORIES);
 		fprintf(ofp, "#define WIDTH_LOCALES\t\t\t%d\n", 3+CATEGORIES);
 		fprintf(ofp, "#define NUM_LOCALES\t\t\t%d\n", num_locales);
@@ -116,11 +121,23 @@ static void do_locale_names(void)
 			fprintf(ofp, "\t%#4x, ", (int)((unsigned char) locales[i].dot_cs));
 			fprintf(ofp, "\t%#4x, ", (int)((unsigned char) locales[i].cs));
 			/* lc_ctype would store translit flags and turkish up/low flag. */
-			fprintf(ofp, "%#4x, ", 0); /* place holder for lc_ctype */
+			fprintf(ofp, "%#4x, ", (int)((unsigned char) locales[i].lc_ctype_row));
 			fprintf(ofp, "%#4x, ", (int)((unsigned char) locales[i].lc_numeric_row));
 			fprintf(ofp, "%#4x, ", (int)((unsigned char) locales[i].lc_monetary_row));
 			fprintf(ofp, "%#4x, ", (int)((unsigned char) locales[i].lc_time_row));
+#if 1
+			/* lc_collate */
+			if (strlen(locales[i].glibc_name) >= 5) {
+			    fprintf(ofp, "COL_IDX_%.2s_%.2s, ", locales[i].glibc_name, locales[i].glibc_name+3);
+			} else if (!strcmp(locales[i].glibc_name, "C")) {
+			    fprintf(ofp, "COL_IDX_C    , ");
+			} else {
+			    printf("don't know how to handle COL_IDX_ for %s\n", locales[i].glibc_name);
+			    exit(EXIT_FAILURE);
+			}
+#else
 			fprintf(ofp, "%#4x, ", 0); /* place holder for lc_collate */
+#endif
 			fprintf(ofp, "%#4x, ", (int)((unsigned char) locales[i].lc_messages_row));
 			fprintf(ofp, "\t/* %s */\n", locales[i].glibc_name);
 		}
@@ -210,722 +227,6 @@ static void do_locale_names(void)
 
 }
 
-int main(int argc, char **argv)
-{
-	if ((argc != 2) || (!(fp = fopen(*++argv, "r")))) {
-		printf("error: missing filename or file!\n");
-		return EXIT_FAILURE;
-	}
-
-	at_strings_end = at_strings;
-
-	read_at_mappings();
-	read_enable_disable();
-	read_locale_list();
-
-	fclose(fp);
-
-	/* handle C locale specially */
-	qsort(locales+1, num_locales-1, sizeof(locale_entry), le_cmp);
-
-#if 0
-	for (i=0 ; i < num_locales ; i++) {
-		printf("locale: %5.5s %2d %2d %s\n",
-			   locales[i].name,
-			   locales[i].cs,
-			   locales[i].dot_cs,
-			   locales[i].glibc_name
-			   );
-	}
-#endif
-
-	if (!(ofp = fopen("locale_tables.h", "w"))) {
-		printf("error: can not open locale_tables.h for writing!\n");
-		return EXIT_FAILURE;
-	}
-
-	do_lc_time();
-	do_lc_numeric();
-	do_lc_monetary();
-	do_lc_messages();
-
-	do_locale_names();
-
-	fclose(ofp);
-
-	printf("total data size = %d\n", total_size);
-	printf("null count = %d\n", null_count);
-
-	return EXIT_SUCCESS;
-}
-
-static char *idx[10000];
-static char buf[100000];
-static char *last;
-static int uniq;
-
-static int addstring(const char *s)
-{
-	int j;
-	size_t l;
-
-	if (!s) {
-		++null_count;
-		return 0;
-	}
-
-	for (j=0 ; j < uniq ; j++) {
-		if (!strcmp(s, idx[j])) {
-			return idx[j] - buf;
-		}
-	}
-	if (uniq >= sizeof(idx)) {
-		printf("too many uniq strings!\n");
-		exit(EXIT_FAILURE);
-	}
-	l = strlen(s) + 1;
-	if (last + l >= buf + sizeof(buf)) {
-		printf("need to increase size of buf!\n");
-		exit(EXIT_FAILURE);
-	}
-
-	idx[uniq] = last;
-	++uniq;
-	strcpy(last, s);
-	last += l;
-	return idx[uniq - 1] - buf;
-}
-
-static int buf16[50*256];
-
-
-static int lc_time_item[50][256];
-static int lc_time_count[50];
-static unsigned char lc_time_uniq_50[700][50];
-static int lc_time_uniq;
-
-#define DO_NL_S(X)	lc_time_S(X, k++)
-
-static void lc_time_S(int X, int k)
-{
-	int j, m;
-	j = addstring(nl_langinfo(X));
-	for (m=0 ; m < lc_time_count[k] ; m++) {
-		if (lc_time_item[k][m] == j) {
-			break;
-		}
-	}
-	if (m == lc_time_count[k]) { /* new for this nl_item */
-		if (m > 255) {
-			printf("too many nl_item %d entries in lc_time\n", k);
-			exit(EXIT_FAILURE);
-		}
-		lc_time_item[k][m] = j;
-		++lc_time_count[k];
-	}
-/*  	printf("\\x%02x", m); */
-	lc_time_uniq_50[lc_time_uniq][k] = m;
-}
-
-static void do_lc_time(void)
-{
-	int i, k, m;
-
-	last = buf+1;
-	uniq = 1;
-	*buf = 0;
-	*idx = buf;
-
-/*  	printf("processing lc_time..."); */
-	for (i=0 ; i < num_locales ; i++) {
-		k = 0;
-
-/*  		printf(" %d", i); fflush(stdout); */
-		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
-			printf("setlocale(LC_ALL,%s) failed!\n",
-				   locales[i].glibc_name);
-		}
-
-		DO_NL_S(ABDAY_1);
-		DO_NL_S(ABDAY_2);
-		DO_NL_S(ABDAY_3);
-		DO_NL_S(ABDAY_4);
-		DO_NL_S(ABDAY_5);
-		DO_NL_S(ABDAY_6);
-		DO_NL_S(ABDAY_7);
-
-		DO_NL_S(DAY_1);
-		DO_NL_S(DAY_2);
-		DO_NL_S(DAY_3);
-		DO_NL_S(DAY_4);
-		DO_NL_S(DAY_5);
-		DO_NL_S(DAY_6);
-		DO_NL_S(DAY_7);
-
-		DO_NL_S(ABMON_1);
-		DO_NL_S(ABMON_2);
-		DO_NL_S(ABMON_3);
-		DO_NL_S(ABMON_4);
-		DO_NL_S(ABMON_5);
-		DO_NL_S(ABMON_6);
-		DO_NL_S(ABMON_7);
-		DO_NL_S(ABMON_8);
-		DO_NL_S(ABMON_9);
-		DO_NL_S(ABMON_10);
-		DO_NL_S(ABMON_11);
-		DO_NL_S(ABMON_12);
-
-		DO_NL_S(MON_1);
-		DO_NL_S(MON_2);
-		DO_NL_S(MON_3);
-		DO_NL_S(MON_4);
-		DO_NL_S(MON_5);
-		DO_NL_S(MON_6);
-		DO_NL_S(MON_7);
-		DO_NL_S(MON_8);
-		DO_NL_S(MON_9);
-		DO_NL_S(MON_10);
-		DO_NL_S(MON_11);
-		DO_NL_S(MON_12);
-
-		DO_NL_S(AM_STR);
-		DO_NL_S(PM_STR);
-
-		DO_NL_S(D_T_FMT);
-		DO_NL_S(D_FMT);
-		DO_NL_S(T_FMT);
-		DO_NL_S(T_FMT_AMPM);
-		DO_NL_S(ERA);
-
-		DO_NL_S(ERA_YEAR);		/* non SuSv3 */
-		DO_NL_S(ERA_D_FMT);
-		DO_NL_S(ALT_DIGITS);
-		DO_NL_S(ERA_D_T_FMT);
-		DO_NL_S(ERA_T_FMT);
-
-		if (k > 50) {
-			printf("error -- lc_time nl_item count > 50!\n");
-			exit(EXIT_FAILURE);
-		}
-
-		{
-			int r;
-			for (r=0 ; r < lc_time_uniq ; r++) {
-				if (!memcmp(lc_time_uniq_50[lc_time_uniq],
-							lc_time_uniq_50[r], 50)) {
-					break;
-				}
-			}
-			if (r == lc_time_uniq) { /* new locale row */
-				++lc_time_uniq;
-				if (lc_time_uniq > 255) {
-					printf("too many unique lc_time rows!\n");
-					exit(EXIT_FAILURE);
-				}
-			}
-			locales[i].lc_time_row = r;
-		}
-/*  		printf("\n"); */
-	}
-/*  	printf(" done\n"); */
-
-	m = k = 0;
-	for (i=0 ; i < 50 ; i++) {
-		k += lc_time_count[i];
-		if (m < lc_time_count[i]) {
-			m = lc_time_count[i];
-		}
-	}
-	printf("buf-size=%d  uniq=%d  item_offsets=%d  max=%d  rows=%d\n",
-		   (int)(last - buf), uniq, k, m, lc_time_uniq);
-/*  	printf("total = %d * 50 + 2 * (50 + %d) + %d = %d\n", */
-/*  		   num_locales, k, (int)(last - buf), */
-/*  		   num_locales*50 + 2*(50 + k) + (int)(last - buf)); */
-	printf("total = %d + %d * 50 + 2 * (50 + %d) + %d = %d\n",
-		   num_locales, lc_time_uniq, k, (int)(last - buf),
-		   i = num_locales + lc_time_uniq*50 + 2*(50 + k) + (int)(last - buf));
-	total_size += i;
-
-	dump_table8c("__lc_time_data", buf, (int)(last - buf));
-
-	for (i=0 ; i < lc_time_uniq ; i++) {
-		m = locales[i].lc_time_row;
-		for (k=0 ; k < 50 ; k++) {
-			buf[50*i + k] = (char)((unsigned char) lc_time_uniq_50[i][k]);
-		}
-	}
-	dump_table8("__lc_time_rows", buf, lc_time_uniq * 50);
-
-	buf16[0] =0;
-	for (i=0 ; i < 50 - 1 ; i++) {
-		buf16[i+1] = buf16[i] + lc_time_count[i];
-	}
-	dump_table16("__lc_time_item_offsets", buf16, 50);
-
-	m = 0;
-	for (k=0 ; k < 50 ; k++) {
-		for (i=0 ; i < lc_time_count[k] ; i++) {
-			buf16[m] = lc_time_item[k][i];
-			++m;
-		}
-	}
-	dump_table16("__lc_time_item_idx", buf16, m);
-}
-
-static void dump_table8(const char *name, const char *tbl, int len)
-{
-	int i;
-
-	fprintf(ofp, "#define %s_LEN\t\t%d\n", name, len);
-	fprintf(ofp, "static const unsigned char %s[%d] = {", name, len);
-	for (i=0 ; i < len ; i++) {
-		if ((i % 12) == 0) {
-			fprintf(ofp, "\n\t");
-		}
-		fprintf(ofp, "%#4x, ", (int)((unsigned char) tbl[i]));
-	}
-	fprintf(ofp, "\n};\n\n");
-}
-
-#define __C_isdigit(c) \
-	((sizeof(c) == sizeof(char)) \
-	 ? (((unsigned char)((c) - '0')) < 10) \
-	 : (((unsigned int)((c) - '0')) < 10))
-#define __C_isalpha(c) \
-	((sizeof(c) == sizeof(char)) \
-	 ? (((unsigned char)(((c) | 0x20) - 'a')) < 26) \
-	 : (((unsigned int)(((c) | 0x20) - 'a')) < 26))
-#define __C_isalnum(c) (__C_isalpha(c) || __C_isdigit(c))
-
-static void dump_table8c(const char *name, const char *tbl, int len)
-{
-	int i;
-
-	fprintf(ofp, "#define %s_LEN\t\t%d\n", name, len);
-	fprintf(ofp, "static const unsigned char %s[%d] = {", name, len);
-	for (i=0 ; i < len ; i++) {
-		if ((i % 12) == 0) {
-			fprintf(ofp, "\n\t");
-		}
-		if (__C_isalnum(tbl[i]) || (tbl[i] == ' ')) {
-			fprintf(ofp, " '%c', ", (int)((unsigned char) tbl[i]));
-		} else {
-			fprintf(ofp, "%#4x, ", (int)((unsigned char) tbl[i]));
-		}
-	}
-	fprintf(ofp, "\n};\n\n");
-}
-
-static void dump_table16(const char *name, const int *tbl, int len)
-{
-	int i;
-
-	fprintf(ofp, "#define %s_LEN\t\t%d\n", name, len);
-	fprintf(ofp, "static const uint16_t %s[%d] = {", name, len);
-	for (i=0 ; i < len ; i++) {
-		if ((i % 8) == 0) {
-			fprintf(ofp, "\n\t");
-		}
-		if (tbl[i] != (uint16_t) tbl[i]) {
-			printf("error - falls outside uint16 range!\n");
-			exit(EXIT_FAILURE);
-		}
-		fprintf(ofp, "%#6x, ", tbl[i]);
-	}
-	fprintf(ofp, "\n};\n\n");
-}
-
-#undef DO_NL_S
-
-static int lc_numeric_item[3][256];
-static int lc_numeric_count[3];
-static unsigned char lc_numeric_uniq_3[700][3];
-static int lc_numeric_uniq;
-
-#define DO_NL_S(X)	lc_numeric_S(X, k++)
-
-static void lc_numeric_S(int X, int k)
-{
-	int j, m;
-	j = addstring(nl_langinfo(X));
-	for (m=0 ; m < lc_numeric_count[k] ; m++) {
-		if (lc_numeric_item[k][m] == j) {
-			break;
-		}
-	}
-	if (m == lc_numeric_count[k]) { /* new for this nl_item */
-		if (m > 255) {
-			printf("too many nl_item %d entries in lc_numeric\n", k);
-			exit(EXIT_FAILURE);
-		}
-		lc_numeric_item[k][m] = j;
-		++lc_numeric_count[k];
-	}
-/*  	printf("\\x%02x", m); */
-	lc_numeric_uniq_3[lc_numeric_uniq][k] = m;
-}
-
-static void do_lc_numeric(void)
-{
-	int i, k, m;
-
-	last = buf+1;
-	uniq = 1;
-	*buf = 0;
-	*idx = buf;
-
-	for (i=0 ; i < num_locales ; i++) {
-		k = 0;
-
-		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
-			printf("setlocale(LC_ALL,%s) failed!\n",
-				   locales[i].glibc_name);
-		}
-
-		DO_NL_S(RADIXCHAR);		/* DECIMAL_POINT */
-		DO_NL_S(THOUSEP);		/* THOUSANDS_SEP */
-		DO_NL_S(GROUPING);
-
-		if (k > 3) {
-			printf("error -- lc_numeric nl_item count > 3!\n");
-			exit(EXIT_FAILURE);
-		}
-
-		{
-			int r;
-			for (r=0 ; r < lc_numeric_uniq ; r++) {
-				if (!memcmp(lc_numeric_uniq_3[lc_numeric_uniq],
-							lc_numeric_uniq_3[r], 3)) {
-					break;
-				}
-			}
-			if (r == lc_numeric_uniq) { /* new locale row */
-				++lc_numeric_uniq;
-				if (lc_numeric_uniq > 255) {
-					printf("too many unique lc_numeric rows!\n");
-					exit(EXIT_FAILURE);
-				}
-			}
-			locales[i].lc_numeric_row = r;
-		}
-	}
-
-	printf("buf-size=%d  uniq=%d  rows=%d\n",
-		   (int)(last - buf), uniq, lc_numeric_uniq);
-	printf("total = %d + %d * 3 + %d = %d\n",
-		   num_locales, lc_numeric_uniq, (int)(last - buf),
-		   i = num_locales + lc_numeric_uniq*3 + (int)(last - buf));
-	total_size += i;
-
-/*  	printf("buf-size=%d  uniq=%d\n", (int)(last - buf), uniq); */
-
-	dump_table8c("__lc_numeric_data", buf, (int)(last - buf));
-
-
-	for (i=0 ; i < lc_numeric_uniq ; i++) {
-		m = locales[i].lc_numeric_row;
-		for (k=0 ; k < 3 ; k++) {
-			buf[3*i + k] = (char)((unsigned char) lc_numeric_uniq_3[i][k]);
-		}
-	}
-	dump_table8("__lc_numeric_rows", buf, lc_numeric_uniq * 3);
-
-	buf16[0] =0;
-	for (i=0 ; i < 3 - 1 ; i++) {
-		buf16[i+1] = buf16[i] + lc_numeric_count[i];
-	}
-	dump_table16("__lc_numeric_item_offsets", buf16, 3);
-
-	m = 0;
-	for (k=0 ; k < 3 ; k++) {
-		for (i=0 ; i < lc_numeric_count[k] ; i++) {
-			buf16[m] = lc_numeric_item[k][i];
-			++m;
-		}
-	}
-	dump_table16("__lc_numeric_item_idx", buf16, m);
-}
-
-#undef DO_NL_S
-
-/*  #define NUM_NL_MONETARY 7 */
-#define NUM_NL_MONETARY (7+14+1)
-
-static int lc_monetary_item[NUM_NL_MONETARY][256];
-static int lc_monetary_count[NUM_NL_MONETARY];
-static unsigned char lc_monetary_uniq_X[700][NUM_NL_MONETARY];
-static int lc_monetary_uniq;
-
-#define DO_NL_S(X)	lc_monetary_S(X, k++)
-
-/*  #define DO_NL_C(X)		printf("%#02x", (int)(unsigned char)(*nl_langinfo(X))); */
-#define DO_NL_C(X) lc_monetary_C(X, k++)
-
-static void lc_monetary_C(int X, int k)
-{
-	int j, m;
-	char c_buf[2];
-
-#warning fix the char entries for monetary... target signedness of char may be different!
-
-	c_buf[1] = 0;
-	c_buf[0] = *nl_langinfo(X);
-	j = addstring(c_buf);
-	for (m=0 ; m < lc_monetary_count[k] ; m++) {
-		if (lc_monetary_item[k][m] == j) {
-			break;
-		}
-	}
-	if (m == lc_monetary_count[k]) { /* new for this nl_item */
-		if (m > 255) {
-			printf("too many nl_item %d entries in lc_monetary\n", k);
-			exit(EXIT_FAILURE);
-		}
-		lc_monetary_item[k][m] = j;
-		++lc_monetary_count[k];
-	}
-/*  	printf("\\x%02x", m); */
-	lc_monetary_uniq_X[lc_monetary_uniq][k] = m;
-}
-
-
-static void lc_monetary_S(int X, int k)
-{
-	int j, m;
-	j = addstring(nl_langinfo(X));
-	for (m=0 ; m < lc_monetary_count[k] ; m++) {
-		if (lc_monetary_item[k][m] == j) {
-			break;
-		}
-	}
-	if (m == lc_monetary_count[k]) { /* new for this nl_item */
-		if (m > 255) {
-			printf("too many nl_item %d entries in lc_monetary\n", k);
-			exit(EXIT_FAILURE);
-		}
-		lc_monetary_item[k][m] = j;
-		++lc_monetary_count[k];
-	}
-/*  	printf("\\x%02x", m); */
-	lc_monetary_uniq_X[lc_monetary_uniq][k] = m;
-}
-
-static void do_lc_monetary(void)
-{
-	int i, k, m;
-
-	last = buf+1;
-	uniq = 1;
-	*buf = 0;
-	*idx = buf;
-
-	for (i=0 ; i < num_locales ; i++) {
-		k = 0;
-
-		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
-			printf("setlocale(LC_ALL,%s) failed!\n",
-				   locales[i].glibc_name);
-		}
-
-
-		/* non SUSv3 */
-		DO_NL_S(INT_CURR_SYMBOL);
-		DO_NL_S(CURRENCY_SYMBOL);
-		DO_NL_S(MON_DECIMAL_POINT);
-		DO_NL_S(MON_THOUSANDS_SEP);
-		DO_NL_S(MON_GROUPING);
-		DO_NL_S(POSITIVE_SIGN);
-		DO_NL_S(NEGATIVE_SIGN);
-		DO_NL_C(INT_FRAC_DIGITS);
-		DO_NL_C(FRAC_DIGITS);
-		DO_NL_C(P_CS_PRECEDES);
-		DO_NL_C(P_SEP_BY_SPACE);
-		DO_NL_C(N_CS_PRECEDES);
-		DO_NL_C(N_SEP_BY_SPACE);
-		DO_NL_C(P_SIGN_POSN);
-		DO_NL_C(N_SIGN_POSN);
-		DO_NL_C(INT_P_CS_PRECEDES);
-		DO_NL_C(INT_P_SEP_BY_SPACE);
-		DO_NL_C(INT_N_CS_PRECEDES);
-		DO_NL_C(INT_N_SEP_BY_SPACE);
-		DO_NL_C(INT_P_SIGN_POSN);
-		DO_NL_C(INT_N_SIGN_POSN);
-
-		DO_NL_S(CRNCYSTR);		/* CURRENCY_SYMBOL */
-
-/*  		printf("\n"); */
-
-		if (k > NUM_NL_MONETARY) {
-			printf("error -- lc_monetary nl_item count > %d!\n", NUM_NL_MONETARY);
-			exit(EXIT_FAILURE);
-		}
-
-		{
-			int r;
-			for (r=0 ; r < lc_monetary_uniq ; r++) {
-				if (!memcmp(lc_monetary_uniq_X[lc_monetary_uniq],
-							lc_monetary_uniq_X[r], NUM_NL_MONETARY)) {
-					break;
-				}
-			}
-			if (r == lc_monetary_uniq) { /* new locale row */
-				++lc_monetary_uniq;
-				if (lc_monetary_uniq > 255) {
-					printf("too many unique lc_monetary rows!\n");
-					exit(EXIT_FAILURE);
-				}
-			}
-			locales[i].lc_monetary_row = r;
-		}
-	}
-
-	printf("buf-size=%d  uniq=%d  rows=%d\n",
-		   (int)(last - buf), uniq, lc_monetary_uniq);
-	printf("total = %d + %d * %d + %d = %d\n",
-		   num_locales, lc_monetary_uniq, NUM_NL_MONETARY, (int)(last - buf),
-		   i = num_locales + lc_monetary_uniq*NUM_NL_MONETARY + (int)(last - buf));
-	total_size += i;
-
-	dump_table8c("__lc_monetary_data", buf, (int)(last - buf));
-
-	for (i=0 ; i < lc_monetary_uniq ; i++) {
-		m = locales[i].lc_monetary_row;
-		for (k=0 ; k < NUM_NL_MONETARY ; k++) {
-			buf[NUM_NL_MONETARY*i + k] = (char)((unsigned char) lc_monetary_uniq_X[i][k]);
-		}
-	}
-	dump_table8("__lc_monetary_rows", buf, lc_monetary_uniq * NUM_NL_MONETARY);
-
-	buf16[0] =0;
-	for (i=0 ; i < NUM_NL_MONETARY - 1 ; i++) {
-		buf16[i+1] = buf16[i] + lc_monetary_count[i];
-	}
-	dump_table16("__lc_monetary_item_offsets", buf16, NUM_NL_MONETARY);
-
-	m = 0;
-	for (k=0 ; k < NUM_NL_MONETARY ; k++) {
-		for (i=0 ; i < lc_monetary_count[k] ; i++) {
-			buf16[m] = lc_monetary_item[k][i];
-			++m;
-		}
-	}
-	dump_table16("__lc_monetary_item_idx", buf16, m);
-}
-
-
-#undef DO_NL_S
-
-static int lc_messages_item[2][256];
-static int lc_messages_count[2];
-static unsigned char lc_messages_uniq_2[700][2];
-static int lc_messages_uniq;
-
-#define DO_NL_S(X)	lc_messages_S(X, k++)
-
-static void lc_messages_S(int X, int k)
-{
-	int j, m;
-	j = addstring(nl_langinfo(X));
-	for (m=0 ; m < lc_messages_count[k] ; m++) {
-		if (lc_messages_item[k][m] == j) {
-			break;
-		}
-	}
-	if (m == lc_messages_count[k]) { /* new for this nl_item */
-		if (m > 255) {
-			printf("too many nl_item %d entries in lc_messages\n", k);
-			exit(EXIT_FAILURE);
-		}
-		lc_messages_item[k][m] = j;
-		++lc_messages_count[k];
-	}
-/*  	printf("\\x%02x", m); */
-	lc_messages_uniq_2[lc_messages_uniq][k] = m;
-}
-
-static void do_lc_messages(void)
-{
-	int i, k, m;
-
-	last = buf+1;
-	uniq = 1;
-	*buf = 0;
-	*idx = buf;
-
-	for (i=0 ; i < num_locales ; i++) {
-		k = 0;
-
-		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
-			printf("setlocale(LC_ALL,%s) failed!\n",
-				   locales[i].glibc_name);
-		}
-
-		DO_NL_S(YESEXPR);
-		DO_NL_S(NOEXPR);
-
-		if (k > 2) {
-			printf("error -- lc_messages nl_item count > 2!\n");
-			exit(EXIT_FAILURE);
-		}
-
-		{
-			int r;
-			for (r=0 ; r < lc_messages_uniq ; r++) {
-				if (!memcmp(lc_messages_uniq_2[lc_messages_uniq],
-							lc_messages_uniq_2[r], 2)) {
-					break;
-				}
-			}
-			if (r == lc_messages_uniq) { /* new locale row */
-				++lc_messages_uniq;
-				if (lc_messages_uniq > 255) {
-					printf("too many unique lc_messages rows!\n");
-					exit(EXIT_FAILURE);
-				}
-			}
-			locales[i].lc_messages_row = r;
-		}
-	}
-
-	printf("buf-size=%d  uniq=%d  rows=%d\n",
-		   (int)(last - buf), uniq, lc_messages_uniq);
-	printf("total = %d + %d * 2 + %d = %d\n",
-		   num_locales, lc_messages_uniq, (int)(last - buf),
-		   i = num_locales + lc_messages_uniq*2 + (int)(last - buf));
-	total_size += i;
-
-/*  	printf("buf-size=%d  uniq=%d\n", (int)(last - buf), uniq); */
-
-	dump_table8c("__lc_messages_data", buf, (int)(last - buf));
-
-	for (i=0 ; i < lc_messages_uniq ; i++) {
-		m = locales[i].lc_messages_row;
-		for (k=0 ; k < 2 ; k++) {
-			buf[2*i + k] = (char)((unsigned char) lc_messages_uniq_2[i][k]);
-		}
-	}
-	dump_table8("__lc_messages_rows", buf, lc_messages_uniq * 2);
-
-	buf16[0] =0;
-	for (i=0 ; i < 2 - 1 ; i++) {
-		buf16[i+1] = buf16[i] + lc_messages_count[i];
-	}
-	dump_table16("__lc_messages_item_offsets", buf16, 2);
-
-	m = 0;
-	for (k=0 ; k < 2 ; k++) {
-		for (i=0 ; i < lc_messages_count[k] ; i++) {
-			buf16[m] = lc_messages_item[k][i];
-			++m;
-		}
-	}
-	dump_table16("__lc_messages_item_idx", buf16, m);
-}
-
-
-
-
-
 static void read_at_mappings(void)
 {
 	char *p;
@@ -1252,3 +553,767 @@ static int le_cmp(const void *a, const void *b)
 	return r;
 }
 
+int main(int argc, char **argv)
+{
+	if ((argc != 2) || (!(fp = fopen(*++argv, "r")))) {
+		printf("error: missing filename or file!\n");
+		return EXIT_FAILURE;
+	}
+
+	at_strings_end = at_strings;
+
+	read_at_mappings();
+	read_enable_disable();
+	read_locale_list();
+
+	fclose(fp);
+
+	/* handle C locale specially */
+	qsort(locales+1, num_locales-1, sizeof(locale_entry), le_cmp);
+
+#if 0
+	for (i=0 ; i < num_locales ; i++) {
+		printf("locale: %5.5s %2d %2d %s\n",
+			   locales[i].name,
+			   locales[i].cs,
+			   locales[i].dot_cs,
+			   locales[i].glibc_name
+			   );
+	}
+#endif
+
+	if (!(ofp = fopen("locale_tables.h", "w"))) {
+		printf("error: can not open locale_tables.h for writing!\n");
+		return EXIT_FAILURE;
+	}
+
+	do_lc_time();
+	do_lc_numeric();
+	do_lc_monetary();
+	do_lc_messages();
+	do_lc_ctype();
+
+	do_locale_names();
+
+	fclose(ofp);
+
+	printf("total data size = %d\n", total_size);
+	printf("null count = %d\n", null_count);
+
+	return EXIT_SUCCESS;
+}
+
+static char *idx[10000];
+static char buf[100000];
+static char *last;
+static int uniq;
+
+static int addblock(const char *s, size_t n) /* l includes nul terminator */
+{
+	int j;
+
+	if (!s) {
+		++null_count;
+		return 0;
+	}
+
+	for (j=0 ; (j < uniq) && (idx[j] + n < last) ; j++) {
+		if (!memcmp(s, idx[j], n)) {
+			return idx[j] - buf;
+		}
+	}
+	if (uniq >= sizeof(idx)) {
+		printf("too many uniq strings!\n");
+		exit(EXIT_FAILURE);
+	}
+	if (last + n >= buf + sizeof(buf)) {
+		printf("need to increase size of buf!\n");
+		exit(EXIT_FAILURE);
+	}
+
+	idx[uniq] = last;
+	++uniq;
+	memcpy(last, s, n);
+	last += n;
+	return idx[uniq - 1] - buf;
+}
+
+static int addstring(const char *s)
+{
+	int j;
+	size_t l;
+
+	if (!s) {
+		++null_count;
+		return 0;
+	}
+
+	for (j=0 ; j < uniq ; j++) {
+		if (!strcmp(s, idx[j])) {
+			return idx[j] - buf;
+		}
+	}
+	if (uniq >= sizeof(idx)) {
+		printf("too many uniq strings!\n");
+		exit(EXIT_FAILURE);
+	}
+	l = strlen(s) + 1;
+	if (last + l >= buf + sizeof(buf)) {
+		printf("need to increase size of buf!\n");
+		exit(EXIT_FAILURE);
+	}
+
+	idx[uniq] = last;
+	++uniq;
+	strcpy(last, s);
+	last += l;
+	return idx[uniq - 1] - buf;
+}
+
+#define DO_LC_COMMON(CATEGORY) \
+	printf("buf-size=%d  uniq=%d  rows=%d\n", \
+		   (int)(last - buf), uniq, lc_##CATEGORY##_uniq); \
+	printf("total = %d + %d * %d + %d = %d\n", \
+		   num_locales, lc_##CATEGORY##_uniq, NUM_NL_##CATEGORY, (int)(last - buf), \
+		   i = num_locales + lc_##CATEGORY##_uniq*NUM_NL_##CATEGORY + (int)(last - buf)); \
+	total_size += i; \
+	dump_table8c("__lc_" #CATEGORY "_data", buf, (int)(last - buf)); \
+	for (i=0 ; i < lc_##CATEGORY##_uniq ; i++) { \
+		m = locales[i].lc_##CATEGORY##_row; \
+		for (k=0 ; k < NUM_NL_##CATEGORY ; k++) { \
+			buf[NUM_NL_##CATEGORY*i + k] = (char)((unsigned char) lc_##CATEGORY##_uniq_X[i][k]); \
+		} \
+	} \
+	dump_table8("__lc_" #CATEGORY "_rows", buf, lc_##CATEGORY##_uniq * NUM_NL_##CATEGORY); \
+	buf16[0] =0; \
+	for (i=0 ; i < NUM_NL_##CATEGORY - 1 ; i++) { \
+		buf16[i+1] = buf16[i] + lc_##CATEGORY##_count[i]; \
+	} \
+	dump_table16("__lc_" #CATEGORY "_item_offsets", buf16, NUM_NL_##CATEGORY); \
+	m = 0; \
+	for (k=0 ; k < NUM_NL_##CATEGORY ; k++) { \
+		for (i=0 ; i < lc_##CATEGORY##_count[k] ; i++) { \
+			buf16[m] = lc_##CATEGORY##_item[k][i]; \
+			++m; \
+		} \
+	} \
+	dump_table16("__lc_" #CATEGORY "_item_idx", buf16, m);
+
+
+#define DL_LC_LOOPTAIL(CATEGORY) \
+		if (k > NUM_NL_##CATEGORY) { \
+			printf("error -- lc_" #CATEGORY " nl_item count > %d!\n", NUM_NL_##CATEGORY); \
+			exit(EXIT_FAILURE); \
+		} \
+		{ \
+			int r; \
+			for (r=0 ; r < lc_##CATEGORY##_uniq ; r++) { \
+				if (!memcmp(lc_##CATEGORY##_uniq_X[lc_##CATEGORY##_uniq], \
+							lc_##CATEGORY##_uniq_X[r], NUM_NL_##CATEGORY)) { \
+					break; \
+				} \
+			} \
+			if (r == lc_##CATEGORY##_uniq) { /* new locale row */ \
+				++lc_##CATEGORY##_uniq; \
+				if (lc_##CATEGORY##_uniq > 255) { \
+					printf("too many unique lc_" #CATEGORY " rows!\n"); \
+					exit(EXIT_FAILURE); \
+				} \
+			} \
+			locales[i].lc_##CATEGORY##_row = r; \
+		}
+
+
+
+static int buf16[100*256];
+
+static void dump_table8(const char *name, const char *tbl, int len)
+{
+	int i;
+
+	fprintf(ofp, "#define %s_LEN\t\t%d\n", name, len);
+	fprintf(ofp, "static const unsigned char %s[%d] = {", name, len);
+	for (i=0 ; i < len ; i++) {
+		if ((i % 12) == 0) {
+			fprintf(ofp, "\n\t");
+		}
+		fprintf(ofp, "%#4x, ", (int)((unsigned char) tbl[i]));
+	}
+	fprintf(ofp, "\n};\n\n");
+}
+
+#define __C_isdigit(c) \
+	((sizeof(c) == sizeof(char)) \
+	 ? (((unsigned char)((c) - '0')) < 10) \
+	 : (((unsigned int)((c) - '0')) < 10))
+#define __C_isalpha(c) \
+	((sizeof(c) == sizeof(char)) \
+	 ? (((unsigned char)(((c) | 0x20) - 'a')) < 26) \
+	 : (((unsigned int)(((c) | 0x20) - 'a')) < 26))
+#define __C_isalnum(c) (__C_isalpha(c) || __C_isdigit(c))
+
+static void dump_table8c(const char *name, const char *tbl, int len)
+{
+	int i;
+
+	fprintf(ofp, "#define %s_LEN\t\t%d\n", name, len);
+	fprintf(ofp, "static const unsigned char %s[%d] = {", name, len);
+	for (i=0 ; i < len ; i++) {
+		if ((i % 12) == 0) {
+			fprintf(ofp, "\n\t");
+		}
+		if (__C_isalnum(tbl[i]) || (tbl[i] == ' ')) {
+			fprintf(ofp, " '%c', ", (int)((unsigned char) tbl[i]));
+		} else {
+			fprintf(ofp, "%#4x, ", (int)((unsigned char) tbl[i]));
+		}
+	}
+	fprintf(ofp, "\n};\n\n");
+}
+
+static void dump_table16(const char *name, const int *tbl, int len)
+{
+	int i;
+
+	fprintf(ofp, "#define %s_LEN\t\t%d\n", name, len);
+	fprintf(ofp, "static const uint16_t %s[%d] = {", name, len);
+	for (i=0 ; i < len ; i++) {
+		if ((i % 8) == 0) {
+			fprintf(ofp, "\n\t");
+		}
+		if (tbl[i] != (uint16_t) tbl[i]) {
+			printf("error - falls outside uint16 range!\n");
+			exit(EXIT_FAILURE);
+		}
+		fprintf(ofp, "%#6x, ", tbl[i]);
+	}
+	fprintf(ofp, "\n};\n\n");
+}
+
+
+#define NUM_NL_time 50
+
+static int lc_time_item[NUM_NL_time][256];
+static int lc_time_count[NUM_NL_time];
+static unsigned char lc_time_uniq_X[700][NUM_NL_time];
+static int lc_time_uniq;
+
+#define DO_NL_S(X)	lc_time_S(X, k++)
+
+static void lc_time_S(int X, int k)
+{
+	size_t len;
+	int j, m;
+	const char *s = nl_langinfo(X);
+	const char *p;
+	static const char nulbuf[] = "";
+
+	if (X == ALT_DIGITS) {
+		len = 1;
+		if (!s) {
+			s = nulbuf;
+		}
+		if (*s) {
+			p = s;
+			for (j = 0 ; j < 100 ; j++) {
+				while (*p) {
+					++p;
+				}
+				++p;
+			}
+			len = p - s;
+		}
+		j = addblock(s, len);
+/* 		if (len > 1) fprintf(stderr, "alt_digit: called addblock with len %zd\n", len); */
+	} else if (X == ERA) {
+		if (!s) {
+			s = nulbuf;
+		}
+		p = s;
+		while (*p) {
+			while (*p) {
+				++p;
+			}
+			++p;
+		}
+		++p;
+		j = addblock(s, p - s);
+/* 		if (p-s > 1) fprintf(stderr, "era: called addblock with len %d\n", p-s); */
+	} else {
+		j = addstring(s);
+	}
+	for (m=0 ; m < lc_time_count[k] ; m++) {
+		if (lc_time_item[k][m] == j) {
+			break;
+		}
+	}
+	if (m == lc_time_count[k]) { /* new for this nl_item */
+		if (m > 255) {
+			printf("too many nl_item %d entries in lc_time\n", k);
+			exit(EXIT_FAILURE);
+		}
+		lc_time_item[k][m] = j;
+		++lc_time_count[k];
+	}
+	lc_time_uniq_X[lc_time_uniq][k] = m;
+}
+
+static void do_lc_time(void)
+{
+	int i, k, m;
+
+	last = buf+1;
+	uniq = 1;
+	*buf = 0;
+	*idx = buf;
+
+	for (i=0 ; i < num_locales ; i++) {
+		k = 0;
+
+		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
+			printf("setlocale(LC_ALL,%s) failed!\n",
+				   locales[i].glibc_name);
+		}
+
+		DO_NL_S(ABDAY_1);
+		DO_NL_S(ABDAY_2);
+		DO_NL_S(ABDAY_3);
+		DO_NL_S(ABDAY_4);
+		DO_NL_S(ABDAY_5);
+		DO_NL_S(ABDAY_6);
+		DO_NL_S(ABDAY_7);
+
+		DO_NL_S(DAY_1);
+		DO_NL_S(DAY_2);
+		DO_NL_S(DAY_3);
+		DO_NL_S(DAY_4);
+		DO_NL_S(DAY_5);
+		DO_NL_S(DAY_6);
+		DO_NL_S(DAY_7);
+
+		DO_NL_S(ABMON_1);
+		DO_NL_S(ABMON_2);
+		DO_NL_S(ABMON_3);
+		DO_NL_S(ABMON_4);
+		DO_NL_S(ABMON_5);
+		DO_NL_S(ABMON_6);
+		DO_NL_S(ABMON_7);
+		DO_NL_S(ABMON_8);
+		DO_NL_S(ABMON_9);
+		DO_NL_S(ABMON_10);
+		DO_NL_S(ABMON_11);
+		DO_NL_S(ABMON_12);
+
+		DO_NL_S(MON_1);
+		DO_NL_S(MON_2);
+		DO_NL_S(MON_3);
+		DO_NL_S(MON_4);
+		DO_NL_S(MON_5);
+		DO_NL_S(MON_6);
+		DO_NL_S(MON_7);
+		DO_NL_S(MON_8);
+		DO_NL_S(MON_9);
+		DO_NL_S(MON_10);
+		DO_NL_S(MON_11);
+		DO_NL_S(MON_12);
+
+		DO_NL_S(AM_STR);
+		DO_NL_S(PM_STR);
+
+		DO_NL_S(D_T_FMT);
+		DO_NL_S(D_FMT);
+		DO_NL_S(T_FMT);
+		DO_NL_S(T_FMT_AMPM);
+		DO_NL_S(ERA);
+
+		DO_NL_S(ERA_YEAR);		/* non SuSv3 */
+		DO_NL_S(ERA_D_FMT);
+		DO_NL_S(ALT_DIGITS);
+		DO_NL_S(ERA_D_T_FMT);
+		DO_NL_S(ERA_T_FMT);
+
+		DL_LC_LOOPTAIL(time)
+	}
+
+	DO_LC_COMMON(time)
+}
+
+#undef DO_NL_S
+
+#define NUM_NL_numeric 3
+
+static int lc_numeric_item[NUM_NL_numeric][256];
+static int lc_numeric_count[NUM_NL_numeric];
+static unsigned char lc_numeric_uniq_X[700][NUM_NL_numeric];
+static int lc_numeric_uniq;
+
+#define DO_NL_S(X)	lc_numeric_S(X, k++)
+
+static void lc_numeric_S(int X, int k)
+{
+	int j, m;
+	char buf[256];
+	char *e;
+	char *s;
+	char c;
+
+	s = nl_langinfo(X);
+	if (X == GROUPING) {
+		if (s) {
+			if ((*s == CHAR_MAX) || (*s == -1)) { /* stupid glibc... :-( */
+				s = "";
+			}
+			e = s;
+			c = 0;
+			while (*e) { /* find end of string */
+				if (*e == CHAR_MAX) {
+					c = CHAR_MAX;
+					++e;
+					break;
+				}
+				++e;
+			}
+			if ((e - s) > sizeof(buf)) {
+				printf("grouping specifier too long\n");
+				exit(EXIT_FAILURE);
+			}
+			strncpy(buf, s, (e-s));
+			e = buf + (e-s);
+			*e = 0;				/* Make sure we're null-terminated. */
+
+			if (c != CHAR_MAX) { /* remove duplicate repeats */
+				while (e > buf) {
+					--e;
+					if (*e != e[-1]) {
+						break;
+					}
+				}
+				*++e = 0;
+			}
+			s = buf;
+		}
+	}
+	j = addstring(s);
+	for (m=0 ; m < lc_numeric_count[k] ; m++) {
+		if (lc_numeric_item[k][m] == j) {
+			break;
+		}
+	}
+	if (m == lc_numeric_count[k]) { /* new for this nl_item */
+		if (m > 255) {
+			printf("too many nl_item %d entries in lc_numeric\n", k);
+			exit(EXIT_FAILURE);
+		}
+		lc_numeric_item[k][m] = j;
+		++lc_numeric_count[k];
+	}
+/*  	printf("\\x%02x", m); */
+	lc_numeric_uniq_X[lc_numeric_uniq][k] = m;
+}
+
+static void do_lc_numeric(void)
+{
+	int i, k, m;
+
+	last = buf+1;
+	uniq = 1;
+	*buf = 0;
+	*idx = buf;
+
+	for (i=0 ; i < num_locales ; i++) {
+		k = 0;
+
+		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
+			printf("setlocale(LC_ALL,%s) failed!\n",
+				   locales[i].glibc_name);
+		}
+
+		DO_NL_S(RADIXCHAR);		/* DECIMAL_POINT */
+		DO_NL_S(THOUSEP);		/* THOUSANDS_SEP */
+		DO_NL_S(GROUPING);
+
+		DL_LC_LOOPTAIL(numeric)
+	}
+
+	DO_LC_COMMON(numeric)
+}
+
+#undef DO_NL_S
+
+#define NUM_NL_monetary (7+14+1)
+
+static int lc_monetary_item[NUM_NL_monetary][256];
+static int lc_monetary_count[NUM_NL_monetary];
+static unsigned char lc_monetary_uniq_X[700][NUM_NL_monetary];
+static int lc_monetary_uniq;
+
+#define DO_NL_S(X)	lc_monetary_S(X, k++)
+
+/*  #define DO_NL_C(X)		printf("%#02x", (int)(unsigned char)(*nl_langinfo(X))); */
+#define DO_NL_C(X) lc_monetary_C(X, k++)
+
+static void lc_monetary_C(int X, int k)
+{
+	int j, m;
+	char c_buf[2];
+
+#warning fix the char entries for monetary... target signedness of char may be different!
+
+	c_buf[1] = 0;
+	c_buf[0] = *nl_langinfo(X);
+	j = addstring(c_buf);
+	for (m=0 ; m < lc_monetary_count[k] ; m++) {
+		if (lc_monetary_item[k][m] == j) {
+			break;
+		}
+	}
+	if (m == lc_monetary_count[k]) { /* new for this nl_item */
+		if (m > 255) {
+			printf("too many nl_item %d entries in lc_monetary\n", k);
+			exit(EXIT_FAILURE);
+		}
+		lc_monetary_item[k][m] = j;
+		++lc_monetary_count[k];
+	}
+/*  	printf("\\x%02x", m); */
+	lc_monetary_uniq_X[lc_monetary_uniq][k] = m;
+}
+
+
+static void lc_monetary_S(int X, int k)
+{
+	int j, m;
+	char buf[256];
+	char *e;
+	char *s;
+	char c;
+
+	s = nl_langinfo(X);
+	if (X == MON_GROUPING) {
+		if (s) {
+			if ((*s == CHAR_MAX) || (*s == -1)) { /* stupid glibc... :-( */
+				s = "";
+			}
+			e = s;
+			c = 0;
+			while (*e) { /* find end of string */
+				if (*e == CHAR_MAX) {
+					c = CHAR_MAX;
+					++e;
+					break;
+				}
+				++e;
+			}
+			if ((e - s) > sizeof(buf)) {
+				printf("mon_grouping specifier too long\n");
+				exit(EXIT_FAILURE);
+			}
+			strncpy(buf, s, (e-s));
+			e = buf + (e-s);
+			*e = 0;				/* Make sure we're null-terminated. */
+
+			if (c != CHAR_MAX) { /* remove duplicate repeats */
+				while (e > buf) {
+					--e;
+					if (*e != e[-1]) {
+						break;
+					}
+				}
+				*++e = 0;
+			}
+			s = buf;
+		}
+	}
+	j = addstring(s);
+	for (m=0 ; m < lc_monetary_count[k] ; m++) {
+		if (lc_monetary_item[k][m] == j) {
+			break;
+		}
+	}
+	if (m == lc_monetary_count[k]) { /* new for this nl_item */
+		if (m > 255) {
+			printf("too many nl_item %d entries in lc_monetary\n", k);
+			exit(EXIT_FAILURE);
+		}
+		lc_monetary_item[k][m] = j;
+		++lc_monetary_count[k];
+	}
+/*  	printf("\\x%02x", m); */
+	lc_monetary_uniq_X[lc_monetary_uniq][k] = m;
+}
+
+static void do_lc_monetary(void)
+{
+	int i, k, m;
+
+	last = buf+1;
+	uniq = 1;
+	*buf = 0;
+	*idx = buf;
+
+	for (i=0 ; i < num_locales ; i++) {
+		k = 0;
+
+		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
+			printf("setlocale(LC_ALL,%s) failed!\n",
+				   locales[i].glibc_name);
+		}
+
+
+		/* non SUSv3 */
+		DO_NL_S(INT_CURR_SYMBOL);
+		DO_NL_S(CURRENCY_SYMBOL);
+		DO_NL_S(MON_DECIMAL_POINT);
+		DO_NL_S(MON_THOUSANDS_SEP);
+		DO_NL_S(MON_GROUPING);
+		DO_NL_S(POSITIVE_SIGN);
+		DO_NL_S(NEGATIVE_SIGN);
+		DO_NL_C(INT_FRAC_DIGITS);
+		DO_NL_C(FRAC_DIGITS);
+		DO_NL_C(P_CS_PRECEDES);
+		DO_NL_C(P_SEP_BY_SPACE);
+		DO_NL_C(N_CS_PRECEDES);
+		DO_NL_C(N_SEP_BY_SPACE);
+		DO_NL_C(P_SIGN_POSN);
+		DO_NL_C(N_SIGN_POSN);
+		DO_NL_C(INT_P_CS_PRECEDES);
+		DO_NL_C(INT_P_SEP_BY_SPACE);
+		DO_NL_C(INT_N_CS_PRECEDES);
+		DO_NL_C(INT_N_SEP_BY_SPACE);
+		DO_NL_C(INT_P_SIGN_POSN);
+		DO_NL_C(INT_N_SIGN_POSN);
+
+		DO_NL_S(CRNCYSTR);		/* CURRENCY_SYMBOL */
+
+		DL_LC_LOOPTAIL(monetary)
+	}
+
+	DO_LC_COMMON(monetary)
+}
+
+
+#undef DO_NL_S
+
+#define NUM_NL_messages 4
+
+static int lc_messages_item[NUM_NL_messages][256];
+static int lc_messages_count[NUM_NL_messages];
+static unsigned char lc_messages_uniq_X[700][NUM_NL_messages];
+static int lc_messages_uniq;
+
+#define DO_NL_S(X)	lc_messages_S(X, k++)
+
+static void lc_messages_S(int X, int k)
+{
+	int j, m;
+	j = addstring(nl_langinfo(X));
+	for (m=0 ; m < lc_messages_count[k] ; m++) {
+		if (lc_messages_item[k][m] == j) {
+			break;
+		}
+	}
+	if (m == lc_messages_count[k]) { /* new for this nl_item */
+		if (m > 255) {
+			printf("too many nl_item %d entries in lc_messages\n", k);
+			exit(EXIT_FAILURE);
+		}
+		lc_messages_item[k][m] = j;
+		++lc_messages_count[k];
+	}
+/*  	printf("\\x%02x", m); */
+	lc_messages_uniq_X[lc_messages_uniq][k] = m;
+}
+
+static void do_lc_messages(void)
+{
+	int i, k, m;
+
+	last = buf+1;
+	uniq = 1;
+	*buf = 0;
+	*idx = buf;
+
+	for (i=0 ; i < num_locales ; i++) {
+		k = 0;
+
+		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
+			printf("setlocale(LC_ALL,%s) failed!\n",
+				   locales[i].glibc_name);
+		}
+
+		DO_NL_S(YESEXPR);
+		DO_NL_S(NOEXPR);
+		DO_NL_S(YESSTR);
+		DO_NL_S(NOSTR);
+
+		DL_LC_LOOPTAIL(messages)
+	}
+
+	DO_LC_COMMON(messages)
+}
+
+#undef DO_NL_S
+
+#define NUM_NL_ctype 10
+
+static int lc_ctype_item[NUM_NL_ctype][256];
+static int lc_ctype_count[NUM_NL_ctype];
+static unsigned char lc_ctype_uniq_X[700][NUM_NL_ctype];
+static int lc_ctype_uniq;
+
+#define DO_NL_S(X)	lc_ctype_S(X, k++)
+
+static void lc_ctype_S(int X, int k)
+{
+	int j, m;
+	j = addstring(nl_langinfo(X));
+	for (m=0 ; m < lc_ctype_count[k] ; m++) {
+		if (lc_ctype_item[k][m] == j) {
+			break;
+		}
+	}
+	if (m == lc_ctype_count[k]) { /* new for this nl_item */
+		if (m > 255) {
+			printf("too many nl_item %d entries in lc_ctype\n", k);
+			exit(EXIT_FAILURE);
+		}
+		lc_ctype_item[k][m] = j;
+		++lc_ctype_count[k];
+	}
+/*  	printf("\\x%02x", m); */
+	lc_ctype_uniq_X[lc_ctype_uniq][k] = m;
+}
+
+static void do_lc_ctype(void)
+{
+	int i, k, m;
+
+	last = buf+1;
+	uniq = 1;
+	*buf = 0;
+	*idx = buf;
+
+	for (i=0 ; i < num_locales ; i++) {
+		k = 0;
+
+		if (!setlocale(LC_ALL, locales[i].glibc_name)) {
+			printf("setlocale(LC_ALL,%s) failed!\n",
+				   locales[i].glibc_name);
+		}
+
+		DO_NL_S(_NL_CTYPE_OUTDIGIT0_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT1_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT2_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT3_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT4_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT5_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT6_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT7_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT8_MB);
+		DO_NL_S(_NL_CTYPE_OUTDIGIT9_MB);
+
+		DL_LC_LOOPTAIL(ctype)
+	}
+
+	DO_LC_COMMON(ctype)
+}
