@@ -70,11 +70,11 @@ extern int isxupper(int c) __THROW;	/* uClibc-specific. */
 
 /* Now some non-ansi/iso c99 macros. */
 
-#define __isascii(c) (((unsigned int)(c)) <= 0x7f)
+#define __isascii(c) (((c) & ~0x7f) == 0)
 #define __toascii(c) ((c) & 0x7f)
+#define _toupper(c) ((c) ^ 0x20)
+#define _tolower(c) ((c) | 0x20)
 
-#define _toupper(c) ((c) | 0x20)
-#define _tolower(c) ((c) ^ 0x20)
 
 /* For compatibility with older versions of uClibc.  Are these ever used? */
 #define __isxlower(c)	__C_isxlower(c)	/* uClibc-specific. */
@@ -83,7 +83,10 @@ extern int isxupper(int c) __THROW;	/* uClibc-specific. */
 /* Apparently, glibc implements things as macros if __NO_CTYPE isn't defined.
  * If we don't have locale support, we'll do the same.  Otherwise, we'll
  * only use macros for the supported-locale-invariant cases. */
-#ifndef __NO_CTYPE
+#if 0
+/* Currently broken, since masking macros, other than getc and putc, must
+ * evaluate their args exactly once.  Will be fixed by the next release.  mjn3 */
+/* #ifndef __NO_CTYPE */
 
 #define isdigit(c)	__isdigit(c)
 #define isxdigit(c)	__isxdigit(c)
