@@ -20,15 +20,32 @@
 
 #include <stdio.h>
 #include <errno.h>
-#include <grp.h>
 #include "config.h"
+    
+#ifdef __UCLIBC_HAS_THREADS__
+#include <pthread.h>
+extern pthread_mutex_t __getgrent_lock;
+# define LOCK   pthread_mutex_lock(&__getgrent_lock)
+# define UNLOCK pthread_mutex_unlock(&__getgrent_lock);
+#else
+# define LOCK
+# define UNLOCK
+#endif
+
+static char *line_buff = NULL;
+static char **members = NULL;
 
 struct group *fgetgrent(FILE * file)
 {
-	if (file == NULL) {
-		__set_errno(EINTR);
-		return NULL;
-	}
+    struct group *grp;
 
-	return __getgrent(fileno(file));
+    if (file == NULL) {
+	__set_errno(EINTR);
+	return NULL;
+    }
+
+    LOCK;
+    grp = __getgrent(fileno(file), line_buff, members);
+    UNLOCK;
+    return grp;
 }
